package eu.clarin.cmdi.oai.harvester;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import eu.clarin.cmdi.oai.harvester.ext.DescriptionParser;

public final class OAIDublinCoreDescriptionParser implements DescriptionParser {
    private static class OAIDublinCoreDescriptionImpl implements
            OAIDublinCoreDescription {
        private final List<String> title;
        private final List<String> creator;
        private final List<String> subject;
        private final List<String> description;
        private final List<String> publisher;
        private final List<String> contributor;
        private final List<String> date;
        private final List<String> type;
        private final List<String> format;
        private final List<String> identifier;
        private final List<String> source;
        private final List<String> language;
        private final List<String> relation;
        private final List<String> coverage;
        private final List<String> rights;

        private OAIDublinCoreDescriptionImpl(Map<String, List<String>> data) {
            this.title = get(data, "title");
            this.creator = get(data, "creators");
            this.subject = get(data, "subjects");
            this.description = get(data, "description");
            this.publisher = get(data, "publisher");
            this.contributor = get(data, "contributor");
            this.date = get(data, "date");
            this.type = get(data, "type");
            this.format = get(data, "format");
            this.identifier = get(data, "identifier");
            this.source = data.get("source");
            this.language = get(data, "language");
            this.relation = get(data, "relation");
            this.coverage = get(data, "coverage");
            this.rights = get(data, "rights");
        }

        @Override
        public String getNamespaceURI() {
            return NAMESPACE_URI;
        }

        @Override
        public String getLocalName() {
            return LOCAL_NAME;
        }

        @Override
        public List<String> getTitle() {
            return title;
        }
        
        @Override
        public List<String> getCreator() {
            return creator;
        }
        
        @Override
        public List<String> getSubject() {
            return subject;
        }
        
        @Override
        public List<String> getDescription() {
            return description;
        }
        
        @Override
        public List<String> getPublishere() {
            return publisher;
        }
        
        @Override
        public List<String> getContributor() {
            return contributor;
        }
        
        @Override
        public List<String> getDate() {
            return date;
        }
        
        @Override
        public List<String> getType() {
            return type;
        }
        
        @Override
        public List<String> getFormat() {
            return format;
        }
        
        @Override
        public List<String> getIdentifier() {
            return identifier;
        }
        
        @Override
        public List<String> getSource() {
            return source;
        }
        
        @Override
        public List<String> getLanguage() {
            return language;
        }
        
        @Override
        public List<String> getRelation() {
            return relation;
        }
        
        @Override
        public List<String> getCoverage() {
            return coverage;
        }
        
        @Override
        public List<String> getRight() {
            return rights;
        }

        public String toString() {
            StringBuilder sb = new StringBuilder("OAIDublinCoreDescription");
            sb.append("[");
            int i = 0;
            if (title != null) {
                sb.append("title=");
                sb.append(title);
                i++;
            }
            if (creator != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("creator=");
                sb.append(creator);
                
            }
            if (subject != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("subject=");
                sb.append(subject);
            }
            if (description != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("descripton=");
                sb.append(description);
            }
            if (publisher != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("publisher=");
                sb.append(publisher);
            }
            if (contributor != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("contributor=");
                sb.append(contributor);
            }
            if (date != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("date=");
                sb.append(date);
            }
            if (type != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("type=");
                sb.append(type);
            }
            if (format != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("format=");
                sb.append(format);
            }
            if (identifier != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("identifier=");
                sb.append(identifier);
            }
            if (source != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("source=");
                sb.append(source);
            }
            if (language != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("language=");
                sb.append(language);
            }
            if (relation != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("relation=");
                sb.append(relation);
            }
            if (coverage != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("coverage=");
                sb.append(coverage);
            }
            if (rights != null) {
                if (i++ > 0) {
                    sb.append(", ");
                }
                sb.append("rights=");
                sb.append(rights);
            }
            sb.append("]");
            return sb.toString();
        }

        private static List<String> get(Map<String, List<String>> data,
                String key) {
            List<String> list = data.get(key);
            if (list != null) {
                if (!list.isEmpty()) {
                    list = Collections.unmodifiableList(list);
                } else {
                    list = null;
                }
            }
            return list;
        }

    } // inner class OAIDublinCoreDescription

    private static final String DC_NS = "http://purl.org/dc/elements/1.1/";

    
    @Override
    public String getNamespaceURI() {
        return OAIDublinCoreDescription.NAMESPACE_URI;
    }

    @Override
    public String getLocalName() {
        return OAIDublinCoreDescription.LOCAL_NAME;
    }

    @Override
    public Description parseDescription(XMLStreamReader reader)
            throws XMLStreamException, HarvesterException {
        Map<String, List<String>> data = null;
        reader.nextTag(); // skip "dc" end tag
        for (;;) {
            if (!DC_NS.equals(reader.getNamespaceURI())) {
                break;
            }
            reader.require(XMLStreamConstants.START_ELEMENT, null, null);
            final String localName = reader.getLocalName()
                .trim().toLowerCase().intern();
            reader.next(); // skip start tag
            final String value = reader.getText().trim();
            reader.next(); // skip content
            reader.require(XMLStreamConstants.END_ELEMENT, DC_NS, localName);
            reader.nextTag();
            if (!value.isEmpty()) {
                if (data == null) {
                    data = new HashMap<String, List<String>>();
                }
                List<String> list = data.get(localName);
                if (list == null) {
                    list = new ArrayList<String>();
                    data.put(localName, list);
                }
                list.add(value);
            }
        } // for
        reader.nextTag(); // skip "dc" end tag
        return (data != null) ? new OAIDublinCoreDescriptionImpl(data) : null;
    }

} // class OAIDublinCoreDescriptionParser
