package eu.clarin.cmdi.oai.harvester;

import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import eu.clarin.cmdi.oai.harvester.ext.DescriptionParser;

public final class OAIIdentifierDescriptionParser implements DescriptionParser {
    private static class OAIIdentifierDescriptionImpl implements
            OAIIdentifierDescription {
        private final String repositoryIdentifier;
        private final String delimiter;
        private final String sampleIdentifier;

        private OAIIdentifierDescriptionImpl(String repositoryIdentifier,
                String delimiter, String sampleIdentifier) {
            this.repositoryIdentifier = repositoryIdentifier;
            this.delimiter = delimiter;
            this.sampleIdentifier = sampleIdentifier;
        }

        @Override
        public String getNamespaceURI() {
            return NAMESPACE_URI;
        }

        @Override
        public String getLocalName() {
            return LOCAL_NAME;
        }

        @Override
        public String getScheme() {
            return OAI_IDENTIFIER_SCHEME;
        }

        @Override
        public String getRepositoryIdentifier() {
            return repositoryIdentifier;
        }

        @Override
        public String getDelimiter() {
            return delimiter;
        }

        @Override
        public String getSampleIdentifier() {
            return sampleIdentifier;
        }

        public String toString() {
            StringBuilder sb = new StringBuilder("OAIIdentifierDescription");
            sb.append("[\"");
            sb.append(OAI_IDENTIFIER_SCHEME);
            sb.append("\", \"");
            sb.append(repositoryIdentifier);
            sb.append("\", \"");
            sb.append(delimiter);
            sb.append("\", \"");
            sb.append(sampleIdentifier);
            sb.append("\"]");
            return sb.toString();
        }

    } // inner class OAIIdentifierDescriptionImpl

    private static final String TAG_SCHEME =
        "scheme";
    private static final String TAG_REPOSITORY_IDENTIFIER =
        "repositoryIdentifier";
    private static final String TAG_DELIMTER =
        "delimiter";
    private static final String TAG_SAMPLE_IDENTIFIER =
        "sampleIdentifier";
    private static final String OAI_IDENTIFIER_SCHEME = "oai";

    @Override
    public String getNamespaceURI() {
        return OAIIdentifierDescription.NAMESPACE_URI;
    }

    @Override
    public String getLocalName() {
        return OAIIdentifierDescription.LOCAL_NAME;
    }

    @Override
    public Description parseDescription(XMLStreamReader reader)
            throws XMLStreamException, HarvesterException {
        reader.require(XMLStreamConstants.START_ELEMENT,
                OAIIdentifierDescription.NAMESPACE_URI,
                OAIIdentifierDescription.LOCAL_NAME);
        reader.nextTag(); // skip "oai-identifier" start tag
        reader.require(XMLStreamConstants.START_ELEMENT,
                OAIIdentifierDescription.NAMESPACE_URI,
                TAG_SCHEME);
        reader.next(); // skip "schema" start tag
        final String scheme = readText(TAG_SCHEME, reader);
        if (!OAI_IDENTIFIER_SCHEME.equals(scheme)) {
            throw new HarvesterException("oai-indentifier scheme must be '" +
                    OAI_IDENTIFIER_SCHEME + "'");
        }
        reader.next(); // skip content
        reader.nextTag(); // "schema" end tag
        reader.require(XMLStreamConstants.START_ELEMENT,
                OAIIdentifierDescription.NAMESPACE_URI,
                TAG_REPOSITORY_IDENTIFIER);
        reader.next(); // skip "repositoryIdentifier" start tag
        final String repositoryIdentifier =
            readText(TAG_REPOSITORY_IDENTIFIER, reader);
        reader.next(); // content
        reader.nextTag(); // skip "repositoryIdentifier" end tag
        reader.require(XMLStreamConstants.START_ELEMENT,
                OAIIdentifierDescription.NAMESPACE_URI,
                TAG_DELIMTER);
        reader.next(); // skip "delimiter" start tag
        final String delimiter = readText(TAG_DELIMTER, reader);
        reader.next(); // advance
        reader.nextTag(); // skip "delimiter" end tag
        reader.require(XMLStreamConstants.START_ELEMENT,
                OAIIdentifierDescription.NAMESPACE_URI,
                TAG_SAMPLE_IDENTIFIER);
        reader.next(); // skip "sampleIdentifier" start tag
        final String sampleIdentifier =
            readText(TAG_SAMPLE_IDENTIFIER, reader);
        reader.next(); // advance
        reader.nextTag(); // skip "sampleIdentifier" end tag
        reader.require(XMLStreamConstants.END_ELEMENT,
                OAIIdentifierDescription.NAMESPACE_URI,
                OAIIdentifierDescription.LOCAL_NAME);
        reader.nextTag(); // skip "oai-identifier" end tag  
        return new OAIIdentifierDescriptionImpl(repositoryIdentifier,
                delimiter, sampleIdentifier);
    }

    private static String readText(String tag, XMLStreamReader reader)
            throws XMLStreamException, HarvesterException {
        String s = reader.getText();
        if (s != null) {
            s = s.trim();
        }
        if ((s == null) || s.isEmpty()) {
            throw new HarvesterException("element '{" +
                    OAIIdentifierDescription.NAMESPACE_URI + "}" + tag +
                    "' must not be empty");
        }
        return s;
    }

} // class OAIIdentifierDescriptionParser
